/*
 * Copyright (c) 2024 ShinGeTsu Meter.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include "hardware/spi.h"

#pragma once

// I2C0 メーター用DAC
#define I2C0_SPEED_kHz 400    // 2000kHzぐらいが限界値
#define I2C0_SDA_PIN   12
#define I2C0_SCL_PIN   13

// I2C1 LCD
#define I2C1_SPEED_kHz 360    // LCDは最大400kHz
#define I2C1_SDA_PIN   10
#define I2C1_SCL_PIN   11

// I2Cアドレス
#define ADDR_MCP4726 0x60
#define ADDR_LCD     0x3e
#define ADDR_ADS1015 0x48

// SPI0 ADC MCP354x
#define SPI0_SPEED_kHz 400
#define SPI0_SCK       18
#define SPI0_MOSI      19
#define SPI0_MISO      16
#define SPI0_CS        17
#define SPI0_BUS       spi0

// DAC
#define DAC_MAX_VALUE 4095.0f
#define DAC_MIN_VALUE 0.0f
#define DAC_WAIT_US   7    // MCP4726の設定時間:6μs + 1μs

// ADC
#define ADC_REFV 2.4f    // デバイス内蔵の2.4v
/**
 * ADC変換待ち時間
 * adcRead()で読み出しを待ち合わせるため、データレートよりオーバーヘッド分短くする
 */
#define ADC_WAIT_US ((1000 * 1000 / 60) - 10)

void    initDevice();
void    adcStart();
void    adcStop();
float   adcRead();
int16_t meterDacSet(float value);
void    lcdPrint(uint8_t row, char *mess);
