/*
 * Copyright (c) 2024 ShinGeTsu Meter.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#pragma once

// GPOIO
#define LED_PIN 25

// ロータリーエンコーダ
#define TA_RE1 26
#define TA_RE2 27

// ボタン
#define BTN1 2
#define BTN2 3
#define BTN3 4
#define BTN4 5
#define BTN5 6
#define BTN6 7
#define BTN7 8
#define BTN8 9
#define BTN9 14

#define BUTTON_SET   BTN1    // TAセット
#define BUTTON_AUTO  BTN2    // 自動モード切替
#define BUTTON_COUNT BTN3    // TAカウントモード切替
#define BUTTON_POWER BTN4    // 電源表示

#define BUTTON_SENS0   BTN5    // TAセット2
#define BUTTON_CLEAR   BTN6    // TAカウンタークリア
#define BUTTON_BACK    BTN7    // TAカウンターを履歴から戻す
#define BUTTON_FORWARD BTN8    // 戻したTAカウンターを進める

#define BUTTON_SET2 BTN9    // TAセット2

// 内蔵ADC
#define VSYS_ADC_PIN 29
#define VSYS_ADC_NO  3

// USB接続
#define VBUS_SENSE_PIN 24

// ロータリーエンコーダ移動方向判定用
#define ROTARY_DIRECTION \
    {0, 1, -1, 0, -1, 0, 0, 1, 1, 0, 0, -1, 0, -1, 1, 0}

// 時間
#define SAMPLING_uS        (100 * 1000)     // 連続サンプリング時間 → イベントチェック間隔
#define UPDATE_uS          (500 * 1000)     // LCD更新間隔・・・0.5秒
#define EVENT_WAIT_uS      (100 * 1000)     // 同じイベントの連続実行を抑止する時間
#define POWER_VIEW_WAIT_uS (2000 * 1000)    // 起動時に電源状態を表示する時間・・・2秒

// 自動セット待ち時間
#define AUTO_SET_WAIT_uS     (500 * 1000)         // デフォルト0.5秒
#define AUTO_SET_WAIT_MAX_uS (3 * 1000 * 1000)    // 最大待ち時間3秒
#define AUTO_SET_ADJ_STEP    (100 * 1000)         // 0.1秒

// 自動セットブロック
#define TA_HIST_uS   (1000 * 1000)                 // TA履歴を保存期間
#define TA_HIST_SIZE (TA_HIST_uS / SAMPLING_uS)    // TA履歴サイズ

// TAカウンタ
#define COUNT_HIST_SIZE 20    // 履歴サイズ

// TA値計算用
#define CONST_V 2.4f        // ブリッジの定電圧値 ・・・ MCP356xRの内部参照電圧
#define CONST_R 21250.0f    // ブリッジの基準抵抗値

// TA値
#define TA_MAX         6.5f       // TA最大値
#define TA_MIN         0.9412f    // TA最小値
#define TA_PAUSE_RANGE 0.800f     // 履歴中の範囲がこれより大きい場合、カウント停止
#define PC_TA_MAX      6.45f      // PCが缶を持っていないと判定する閾値

// 感度
#define SENS_NORM_INITIAL    4.0f      // 通常モードの初期値
#define SENS_LOW_INITIAL     0.5f      // 低感度モード時の初期値
#define SENS_MIN             0.02f     // 感度の最小値
#define SENS_MAX             127.0f    // 感度の最大値
#define SENS_COEF            1.05f     // 感度操作の増減割合
#define CORRECT_TA_BASE      3.0f      // 補正の中心とするTA値
#define CORRECT_COEF_TA2     0.78f     // TA3を1としたときのTA2の時の補正係数
#define CORRECT_TA_LOW_TH    2.6f      // 補正を緩くする低いほうの境界値
#define CORRECT_TA_HIGH_TH   4.3f      // 補正を緩くする高いほうの境界値
#define CORRECT_TA_LOW_RATE  0.1f      // 低いほうの補正を穏やかにする割合
#define CORRECT_TA_HIGH_RATE 0.6f      // 高いほうの補正を穏やかにする割合

// メーター目盛   左端=0, セット位置=15,  右端=40
#define METER_SCALE_SIZE 40.0f    // メーターの目盛数
#define METER_SCALE_SET  15.0f    // セット位置
